#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
    2015/10/14  WeiYanfeng
    公共函数 包

~~~~~~~~~~~~~~~~~~~~~~~~
共函数 包
~~~~~~~~~~~~~~~~~~~~~~~~
# 依赖包 Package required
# pip install redis

"""
# WeiYF.20170605 经测试，Python3引用当前目录下的源码文件，文件名前要加`.`点号
# 同时这样的写法也可以在Python2.7下使用。

from .CRedisSubscribe import GetRedisClient,CRedisSubscribe,RedisPipeWatchExec
from .CAutoConnectRedis import CAutoConnectRedis




#! /usr/local/bin/python
#-*- coding:utf-8 -*-

"""
@author: weber.juche@gmail.com
@time: 2016/12/5 20:47

Redis客户端订阅封装

"""

import sys

import os
import time
import redis

from weberFuncs import PrintTimeMsg, GetCurrentTime, IsPython3, PrintAndSleep


class CLogCmdRedis(redis.StrictRedis):

    def __init__(self, host='localhost', port=6379,
                 db=0, password=None, socket_timeout=None,
                 **kwarg):
        redis.StrictRedis.__init__(self, host=host, port=port,
                                   db=db, password=password,
                                   socket_timeout=socket_timeout,
                                   **kwarg)
        self.sFullPathRedisLogFN = ''  # 默认为空表示不启用

    def SetFullPathRedisLogFN(self, sFullPathRedisLogFN):
        # 设置记录Redis命令的文件名，空串表示不记录
        self.sFullPathRedisLogFN = sFullPathRedisLogFN

    def execute_command(self, *args, **options):
        oRet = redis.StrictRedis.execute_command(self, *args, **options)
        # PrintTimeMsg('exec(%s)=%s!' % (' '.join(args),str(oRet)) )
        if self.sFullPathRedisLogFN:
            with open(self.sFullPathRedisLogFN, "a") as f:  # 追加模式输出
                lsS = []
                for arg in args:
                    lsS.append(str(arg))
                sLogMsg = 'exec(%s)=%s!' % (' '.join(lsS),str(oRet))
                sS = "[%s]%s\n" % (GetCurrentTime(), sLogMsg)
                f.write(sS)
        return oRet


def GetRedisClient(sRedisParam):
    # try:
    #     import redis
    # except ImportError as e:
    #     import traceback
    #     traceback.print_exc()
    # sRedisParam = '192.168.2.209:6379:6'
    # sRedisParam = '192.168.2.209:6379:6:password'
    redisHOST = '127.0.0.1'
    redisPORT = 6379
    redisDB = 1
    redisPASS = None   # WeiYF.20160414 Redis参数支持密码
    redisTIMEOUT = 120  # WeiYF.20170322 支持超时参数
    lsP = sRedisParam.split(':')
    if len(lsP) >= 2:
        redisHOST = lsP[0]
        redisPORT = int(lsP[1])
        if len(lsP) >= 3:
            redisDB = int(lsP[2])
            if len(lsP) >= 4:
                redisPASS = lsP[3]
                if len(lsP) >= 5:
                    redisTIMEOUT = int(lsP[4])
    oRedis = CLogCmdRedis(host=redisHOST, port=redisPORT, db=redisDB,
                          password=redisPASS,
                          socket_timeout=redisTIMEOUT,  # WeiYF.20160606 新增超时参数
                          decode_responses=True,  # WeiYF.20250731 自动把 bytes 转成 str
                          )
    sRedisParamHint = '%s:%s:%s' % (redisHOST, redisPORT, redisDB)
    PrintTimeMsg("GetRedisClient(%s)ReturnOK..." % sRedisParamHint)
    return oRedis


class CRedisClientBase:

    def __init__(self, sRedisParam=''):
        self.oRedis = GetRedisClient(sRedisParam)

    def __del__(self):
        if self.oRedis:
            self.oRedis.connection_pool.disconnect()
            PrintTimeMsg("CRedisClientBase.disconnect()!!!")

    def TryConnect(self):
        if not self.oRedis:
            self.oRedis = redis.StrictRedis(host=self.redisHOST, port=self.redisPORT,
                                            db=self.redisDB, password=self.redisPASS,
                                            decode_responses=True)
            sMsg = "{%d}TryConnect.connect(%s)..." % (os.getpid(), self.sRedisParamHint)
            PrintTimeMsg(sMsg)


class CRedisSubscribe(CRedisClientBase):

    def __init__(self, sRedisParam=''):
        CRedisClientBase.__init__(self, sRedisParam)
        self.oPubSub = None
        self.bLoopRunFlag = True  # WeiYF.20150514 循环运行标记，为False时退出订阅循环

    def __del__(self):
        if self.oPubSub:
            self.oPubSub.close()
        CRedisClientBase.__del__(self)

    def SubscribeAndLoop(self, sSubKey, ftCallBack, iTimeOutSeconds=60, ftCBTimeOut=None):
        self.oPubSub = None
        iLoopCnt = 0
        tmLastGetMsg = 0
        tmLastTimeOut = time.time()  # 订阅后首次超时也退出
        while self.bLoopRunFlag:
            if not self.oPubSub:
                self.TryConnect()
                self.oPubSub = self.oRedis.pubsub()
                if '*' in sSubKey:
                    self.oPubSub.psubscribe(sSubKey)
                else:
                    self.oPubSub.subscribe(sSubKey)
                sMsg = 'Subscribe(%s)' % str(sSubKey)
                # LogCriticalMsg(sMsg)
                PrintTimeMsg('oPubSub.' + sMsg + '...')
            if not self.oPubSub.connection:
                PrintTimeMsg('oPubSub.connection=None...')
                if self.oPubSub:
                    self.oPubSub.close()
                    self.oPubSub = None
                time.sleep(0.001)  # be nice to the system :)
                continue
            iLoopCnt += 1
            # msg = ''
            try:
                # if self.oPubSub and self.oPubSub.connection:
                # WeiYF.20150608 应该是不判断链接状态，才会出异常
                # WeiYF.20150609 经测试，不判断链接状态，不会出异常
                msg = self.oPubSub.get_message()  # No Block 不阻塞
            except Exception as e:
                PrintTimeMsg('oPubSub.Exception.e=(%s)Continue...' % (str(e)))
                if self.oPubSub:
                    self.oPubSub.close()
                    self.oPubSub = None
                time.sleep(0.001)  # be nice to the system :)
                continue
            if msg:
                # PrintTimeMsg('oPubSub.msg=%s=' % msg)
                sType = msg.get('type', '')
                if sType == 'subscribe':
                    PrintTimeMsg('oPubSub.SubscribeReturn=(%s)' % str(msg))
                elif sType == 'message' or sType == 'pmessage':
                    # PrintTimeMsg("SubscribeAndLoop.iLoopCnt=%s,msg=%s=" % (iLoopCnt, msg))
                    sChannel = msg.get('channel', b'')
                    sData = msg.get('data', b'')
                    # PrintTimeMsg(f"SubscribeAndLoop.type(sChannel)={type(sChannel)},type(sData)={type(sData)}=")
                    if sData:  # and type(sData) == str
                        # PrintTimeMsg("SubscribeAndLoop.iLoopCnt=%s,RcvData=(%s)" % (iLoopCnt,str(sData)) )
                        # if IsPython3() and sData is bytes:
                        #     # sData = sData.decode('utf8')
                        #     pass
                        #     # PrintTimeMsg("SubscribeAndLoop.iLoopCnt=%s,sData=(%s)" % (iLoopCnt, sData))
                        # else:
                        #     sData = str(sData)
                        # if IsPython3():
                        #     sChannel = sChannel.decode('utf8')
                        if sChannel is bytes:
                            sChannel = sChannel.decode('unicode_escape')  # bytes -> str
                        if sData is bytes:
                            sData = sData.decode('unicode_escape')  # bytes -> str
                        ftCallBack(sChannel, sData)
                        tmLastTimeOut = time.time()
                    else:
                        PrintTimeMsg("SubscribeAndLoop.iLoopCnt=%s,Receive=(%s)" % (iLoopCnt, str(msg)))
                time.sleep(0.01)  # be nice to the system :)
                continue
            else:
                if ftCBTimeOut and time.time() - tmLastTimeOut > iTimeOutSeconds:
                    # 超时回调 #and tmLastTimeOut>0
                    # 调用 ftCBTimeOut 首次也要调用，否则会出现没有数据情况下吊死
                    ftCBTimeOut()
                    tmLastTimeOut = time.time()
                if time.time() - tmLastGetMsg > 60:  # 60秒主动检查一次
                    if sSubKey is str and sSubKey.startswith('LIST_'):
                        # 若是列表型，则主动去查询列表
                        lsData = self.oRedis.lrange(sSubKey, 0, 0)  # 取第一个
                        if len(lsData) >= 1:
                            sData = lsData[0]
                            PrintTimeMsg("SubscribeAndLoop.iLoopCnt=%s,GetData=(%s)" % (iLoopCnt, str(sData)))
                            ftCallBack('@LIST', sData)
                            tmLastTimeOut = time.time()
                            tmLastGetMsg = tmLastTimeOut
                sleepSeconds = 0.1  # 0.01
                time.sleep(sleepSeconds)  # be nice to the system :)


def mainCRedisSubscribe():
    # o = CRedisSubscribe()
    o = CRedisSubscribe('localside:6379:5')

    def cbPrint(sChannel, sData):
        PrintTimeMsg("cbPrint.sChannel=%s,sData=%s=" % (sChannel, sData))
    # o.SubscribeAndLoop('ME:TEST',cbPrint)
    o.SubscribeAndLoop('*', cbPrint)

# ------------------------------


def RedisPipeWatchExec(oRedis, lsWatchKeys, ftCallBackGet, ftCallBackSet, bVerbose=True):
    with oRedis.pipeline() as pipe:
        iLoopCnt = 0
        while True:
            iLoopCnt += 1
            try:
                # put a WATCH on the key that holds our sequence value
                pipe.watch(*lsWatchKeys)
                # after WATCHing, the pipeline is put into immediate execution
                # mode until we tell it to start buffering commands again.
                # this allows us to get the current value of our sequence
                # # current_value = pipe.get('OUR-SEQUENCE-KEY')
                # # next_value = int(current_value) + 1
                if not ftCallBackGet(pipe):
                    PrintTimeMsg('RedisPipeWatchExec.#%d.ftCallBackGet()=Error!' % iLoopCnt)
                    return False, 'GetError'
                # now we can put the pipeline back into buffered mode with MULTI
                pipe.multi()
                # # pipe.set('OUR-SEQUENCE-KEY', next_value)
                if ftCallBackSet(pipe):
                    # and finally, execute the pipeline (the set command)
                    oRet = pipe.execute()
                    if bVerbose:
                        PrintTimeMsg('RedisPipeWatchExec.#%d.execute().OUT=%s!' % (iLoopCnt, str(oRet)))
                    return True, oRet
                PrintTimeMsg('RedisPipeWatchExec.#%d.Discard!' % iLoopCnt)
                pipe.reset()
                return False, 'Discard'
                # if a WatchError wasn't raised during execution, everything
                # we just did happened atomically.
                # # break
            except redis.WatchError as e:
                # another client must have changed 'OUR-SEQUENCE-KEY' between
                # the time we started WATCHing it and the pipeline's execution.
                # our best bet is to just retry.
                PrintTimeMsg('RedisPipeWatchExec.#%d.WatchError=(%s)=Continue!' % (iLoopCnt, str(e)))
                if iLoopCnt > 5:
                    PrintTimeMsg('RedisPipeWatchExec.iLoopCnt=(%d)>5!' % iLoopCnt)
                    return False, 'LoopOut'
                continue
            except Exception as e:
                PrintTimeMsg('RedisPipeWatchExec.#%d.Exception=(%s)=Discard!' % (iLoopCnt, str(e)))
                pipe.reset()
                return False, 'Exception'


def mainRedisPipeWatchExec():
    r = redis.Redis(host='localhost', port=6379)

    def fGet(p):
        print(p.get('num'))
        return True
        # return False

    def fSet(p):
        p.set('key1', 'value1')
        p.incr('num')
        print(p.command_stack)
        p.pipeline_execute_command('SET', 'key2', 'value2')
        # p.pipeline_execute_command('SET key3 value3')
        print(p.command_stack)
        time.sleep(5)
        return True
        # return False
    RedisPipeWatchExec(r, ['key1', 'num'], fGet, fSet)


# -------------------------------
if __name__ == '__main__':
    mainCRedisSubscribe()
    # mainRedisPipeWatchExec()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-

import codecs
import os
# import sys

try:
    from setuptools import setup  # 打包的用的setup必须引入
except ImportError as e:
    from distutils.core import setup


def loadLongDesc(fname):
    """
    定义一个read方法，用来读取目录下的长描述
    我们一般是将README文件中的内容读取出来作为长描述，这个会在PyPI中你这个包的页面上展现出来，
    你也可以不用这个方法，自己手动写内容即可，
    PyPI上支持.rst格式的文件。暂不支持.md格式的文件，<BR>.rst文件PyPI会自动把它转为HTML形式显示在你包的信息页面上。
    """
    # return codecs.open(os.path.join(os.path.dirname(__file__), fname),
    #                    "r",encoding='utf-8').read()
    # WeiYF.20170605 为避免 pip install 时出现 UnicodeDecodeError 暂时直接使用简单串
    return "weberRedis"


def main_setup():
    setup(
        name="weberRedis",  # 包的名字
        packages=["weberRedis"],  # 包含的包，可以多个，这是一个列表
        version="0.0.14",  # 当前包的版本，这个按你自己需要的版本控制方式来

        keywords="public weberRedis weber",  # 关于当前包的一些关键字，方便PyPI进行分类。
        author="Weber Juche",  # 包的作者
        author_email="weber.juche@gmail.com",  # 作者的邮件地址
        url="https://github.com/Weber-JC/PythonEggs/tree/master/weberFuncs",  # 这个包的项目地址
        license="MIT",  # 授权方式,

        # zip_safe=True,
        zip_safe=False,

        description='Public functions library wrote by Weber Juche.',
        # u"包含了常用的简单函数的共享库", #关于这个包的描述,这个会出现 PyPI 的列表页面

        long_description=loadLongDesc("README.md"),  # 参见loadLongDesc方法说明

        classifiers = [
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python',
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
        ],
        include_package_data=True,

        install_requires=[
            # "requests", # 避免upgrade依赖
        ],  # 所依赖的PyPi包
        # dependency_links=['http://github.com/user/repo/tarball/master#egg=package-1.0'] # 非PyPi包
    )

if __name__ == '__main__':
    main_setup()
    # print(loadLongDesc("README.md"))


#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2015/11/5  WeiYanfeng
    判断链接状态，并自动重连的Redis链接封装类
"""

import sys
import time

from weberFuncs import PrintTimeMsg, GetCurrentTime
from .CRedisSubscribe import GetRedisClient


class CAutoConnectRedis:
    def __init__(self, sClientName):
        self.sClientName = sClientName
        self.dictRedisByParam = {}

    def GetRedis(self, sRedisParam):
        # PrintTimeMsg('GetRedis.sRedisParam=(%s)' % (sRedisParam))
        dictRedis = self.dictRedisByParam.get(sRedisParam, {})
        oRedis = dictRedis.get('oRedis', None)
        bNeedNew = not oRedis
        try:
            if oRedis:
                oRedis.incr('KV_COUNT_COMMAND_%s' % self.sClientName)
                dictRedis['tmLastCommand'] = time.time()
        except Exception as e:  # ConnectionError TimeoutError
            import traceback
            traceback.print_exc()  # WeiYF.20151022 打印异常整个堆栈 这个对于动态加载非常有用
            PrintTimeMsg('GetRedis.COMMAND.Exception.e=(%s)' % (str(e)))
            bNeedNew = True
        if bNeedNew:
            try:
                oRedis = GetRedisClient(sRedisParam)
                oRedis.incr('KV_COUNT_CONNECT_%s' % self.sClientName)
                dictRedis['oRedis'] = oRedis
                dictRedis['tmLastConnect'] = time.time()
                self.dictRedisByParam[sRedisParam] = dictRedis
            except Exception as e:  # ConnectionError TimeoutError
                import traceback
                traceback.print_exc()  # WeiYF.20151022 打印异常整个堆栈 这个对于动态加载非常有用
                PrintTimeMsg('GetRedis.CONNECT.Exception.e=(%s)' % (str(e)))
                return None
        return oRedis


def testCAutoConnectRedis():
    acb = CAutoConnectRedis('Windows')
    oRedis = acb.GetRedis('192.168.2.209:6379:6')
    print(oRedis.hgetall('HASH_Proxy1_8888_STATUS'))
    oRedis = acb.GetRedis('192.168.2.209:6379:6')
    print(oRedis.hgetall('HASH_Proxy1_8888_STATUS'))
    print(oRedis.hgetall('HASH_Proxy1_8899_STATUS'))
    print(acb.dictRedisByParam)

# ------------------------------
if __name__ == '__main__':
    testCAutoConnectRedis()